package reborncore.common.container;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.Container;
import net.minecraft.inventory.Slot;
import net.minecraft.item.ItemStack;
import net.minecraft.tileentity.TileEntity;
import reborncore.api.tile.IContainerLayout;
import reborncore.client.gui.BaseSlot;
import reborncore.client.gui.SlotFake;

import javax.annotation.Nullable;
import java.util.HashMap;
import java.util.List;

public abstract class RebornContainer extends Container
{

	public HashMap<Integer, BaseSlot> slotMap = new HashMap<>();

	@Override
	protected Slot func_75146_a(Slot slotIn) {
		Slot slot = super.func_75146_a(slotIn);
		if(slot instanceof BaseSlot){
			//TODO remove player slots
			slotMap.put(slot.getSlotIndex(), (BaseSlot) slot);
		}
		return slot;
	}

	private static HashMap<String, RebornContainer> containerMap = new HashMap<>();

	public static @Nullable RebornContainer getContainerFromClass(Class<? extends RebornContainer> clazz, TileEntity tileEntity){
		if(containerMap.containsKey(clazz.getCanonicalName())){
			return containerMap.get(clazz.getCanonicalName());
		} else {
			try {
				//TODO think hard about how to fix this one
				RebornContainer container = clazz.newInstance();
				if(container instanceof IContainerLayout){
					((IContainerLayout) container).setTile(tileEntity);
					((IContainerLayout) container).addInventorySlots();
				}
				containerMap.put(clazz.getCanonicalName(), container);
				return container;
			} catch (InstantiationException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			}
		}
		return null;
	}

	public static RebornContainer createContainer(Class<? extends RebornContainer> clazz, TileEntity tile, EntityPlayer player){
		try {
			RebornContainer container = clazz.newInstance();
			if(container instanceof IContainerLayout){
				((IContainerLayout) container).setPlayer(player);
				((IContainerLayout) container).setTile(tile);
				((IContainerLayout) container).addInventorySlots();
				((IContainerLayout) container).addPlayerSlots();
			}
			return container;
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public ItemStack func_82846_b(EntityPlayer player, int slotIndex)
	{
		ItemStack originalStack = null;
		Slot slot = (Slot) field_75151_b.get(slotIndex);
		int numSlots = field_75151_b.size();
		if (slot != null && slot.func_75216_d())
		{
			ItemStack stackInSlot = slot.func_75211_c();
			originalStack = stackInSlot.func_77946_l();
			if (slotIndex >= numSlots - 9 * 4 && tryShiftItem(stackInSlot, numSlots))
			{
				// NOOP
			} else if (slotIndex >= numSlots - 9 * 4 && slotIndex < numSlots - 9)
			{
				if (!shiftItemStack(stackInSlot, numSlots - 9, numSlots))
				{
					return null;
				}
			} else if (slotIndex >= numSlots - 9 && slotIndex < numSlots)
			{
				if (!shiftItemStack(stackInSlot, numSlots - 9 * 4, numSlots - 9))
				{
					return null;
				}
			} else if (!shiftItemStack(stackInSlot, numSlots - 9 * 4, numSlots))
			{
				return null;
			}
			slot.func_75220_a(stackInSlot, originalStack);
			if (stackInSlot.field_77994_a <= 0)
			{
				slot.func_75215_d(null);
			} else
			{
				slot.func_75218_e();
			}
			if (stackInSlot.field_77994_a == originalStack.field_77994_a)
			{
				return null;
			}
			slot.func_82870_a(player, stackInSlot);
		}
		return originalStack;
	}

	protected boolean shiftItemStack(ItemStack stackToShift, int start, int end)
	{
		boolean changed = false;
		if (stackToShift.func_77985_e())
		{
			for (int slotIndex = start; stackToShift.field_77994_a > 0 && slotIndex < end; slotIndex++)
			{
				Slot slot = (Slot) field_75151_b.get(slotIndex);
				ItemStack stackInSlot = slot.func_75211_c();
				if (stackInSlot != null && canStacksMerge(stackInSlot, stackToShift))
				{
					int resultingStackSize = stackInSlot.field_77994_a + stackToShift.field_77994_a;
					int max = Math.min(stackToShift.func_77976_d(), slot.func_75219_a());
					if (resultingStackSize <= max)
					{
						stackToShift.field_77994_a = 0;
						stackInSlot.field_77994_a = resultingStackSize;
						slot.func_75218_e();
						changed = true;
					} else if (stackInSlot.field_77994_a < max)
					{
						stackToShift.field_77994_a -= max - stackInSlot.field_77994_a;
						stackInSlot.field_77994_a = max;
						slot.func_75218_e();
						changed = true;
					}
				}
			}
		}
		if (stackToShift.field_77994_a > 0)
		{
			for (int slotIndex = start; stackToShift.field_77994_a > 0 && slotIndex < end; slotIndex++)
			{
				Slot slot = (Slot) field_75151_b.get(slotIndex);
				ItemStack stackInSlot = slot.func_75211_c();
				if (stackInSlot == null)
				{
					int max = Math.min(stackToShift.func_77976_d(), slot.func_75219_a());
					stackInSlot = stackToShift.func_77946_l();
					stackInSlot.field_77994_a = Math.min(stackToShift.field_77994_a, max);
					stackToShift.field_77994_a -= stackInSlot.field_77994_a;
					slot.func_75215_d(stackInSlot);
					slot.func_75218_e();
					changed = true;
				}
			}
		}
		return changed;
	}

	private boolean tryShiftItem(ItemStack stackToShift, int numSlots)
	{
		for (int machineIndex = 0; machineIndex < numSlots - 9 * 4; machineIndex++)
		{
			Slot slot = (Slot) field_75151_b.get(machineIndex);
			if (slot instanceof SlotFake)
			{
				continue;
			}
			if (!slot.func_75214_a(stackToShift))
				continue;
			if (shiftItemStack(stackToShift, machineIndex, machineIndex + 1))
				return true;
		}
		return false;
	}

	public static boolean canStacksMerge(ItemStack stack1, ItemStack stack2)
	{
		if (stack1 == null || stack2 == null)
		{
			return false;
		}
		if (!stack1.func_77969_a(stack2))
		{
			return false;
		}
		if (!ItemStack.func_77970_a(stack1, stack2))
		{
			return false;
		}
		return true;

	}
}
